#include <math.h>
#include <graphics.h>

#include "mys.h"
#include "mat_fce.h"
#include "win_fce.h"
#include "beh_prog.h"

const float PI=3.1415926536;
const float nepres=0.00001;
const float odm3=sqrt(3.0);
const float odm3pol=sqrt(1.5);
const float sinu=sin(ALFA);
const float cosu=cos(ALFA);


bod3d operator-(bod3d vektor)
{
  bod3d vystup;
  vystup.x=-vektor.x;
  vystup.y=-vektor.y;
  vystup.z=-vektor.z;
  return vystup;
}

void koef(float *k, float *l, float *m, float *n, float *jz, float alfa, float beta)
{
  float sina, cosa, sinb, cosb, sinab, cosab;
  float koefx, koefy, koefz;
  sina=sin(alfa);
  sinb=sin(beta);
  cosa=cos(alfa);
  cosb=cos(beta);
  sinab=sin(alfa + beta);
  cosab=cos(alfa + beta);

  koefx=1/(cosa*sqrt(1+((sina*cosb)/(cosa*sinb))));
  koefy=1/(cosb*sqrt(1+((sinb*cosa)/(cosb*sina))));
  *jz=1/(cosa*sqrt(1+((sina*sinab)/(cosa*cosab))));
  *k=-koefx*cosa;
  *l=koefy*cosb;
  *m=-koefx*sina;
  *n=-koefy*sinb;
}

void axonom(bod3d *vstup, bod2d *vystup, int pocet, float alfa, float beta)
{
  int i;
  float K,L,M,N,JZ;
  koef(&K,&L,&M,&N,&JZ,alfa,beta);
  for(i=0;i<pocet;i++)
  {
    (vystup+i)->x = ((vstup+i)->x * K) + ((vstup+i)->y * L);
    (vystup+i)->y = ((vstup+i)->x * M) + ((vstup+i)->y * N) + ((vstup+i)->z * JZ);
  }
}


bod2d kosouhle(bod3d vstup)
{
  bod2d vystup;

/*  vystup.x=vstup.x-vstup.y*ZKRACENI_Y*cosu;
  vystup.y=vstup.z-vstup.y*ZKRACENI_Y*sinu; */ /*Levotociva soustava*/
  vystup.x=vstup.y-vstup.x*ZKRACENI_Y*cosu;    /*Pravotociva soustava*/
  vystup.y=vstup.z-vstup.x*ZKRACENI_Y*sinu;

  return vystup;
}


void zvetseni(bod3d *vstup, bod3d *vystup, int pocet, float meritko)
{
  int i;
  for(i=0;i<pocet;i++)
  {
    (vystup+i)->x = meritko * (vstup+i)->x;
    (vystup+i)->y = meritko * (vstup+i)->y;
    (vystup+i)->z = meritko * (vstup+i)->z;
  }
}

void zvetseni(bod2d *vstup, bod2d *vystup, int pocet, float meritko)
{
  int i;
  for(i=0;i<pocet;i++)
  {
    (vystup+i)->x = meritko * (vstup+i)->x;
    (vystup+i)->y = meritko * (vstup+i)->y;
  }
}


float skal_souc(bod3d v1, bod3d v2)
{
  return v1.x*v2.x+v1.y*v2.y+v1.z*v2.z;
}



bod3d operator*(bod3d v1, bod3d v2)
{
  bod3d vysl;
  vysl.x=v1.y*v2.z-v1.z*v2.y;
  vysl.y=v1.z*v2.x-v1.x*v2.z;
  vysl.z=v1.x*v2.y-v1.y*v2.x;
  return vysl;
}

bod3d operator-(bod3d v1, bod3d v2)
{
  bod3d vysl;
  vysl.x=v1.x-v2.x;
  vysl.y=v1.y-v2.y;
  vysl.z=v1.z-v2.z;
  return vysl;
}


bod3d operator+(bod3d v1, bod3d v2)
{
  bod3d vysl;
  vysl.x=v1.x+v2.x;
  vysl.y=v1.y+v2.y;
  vysl.z=v1.z+v2.z;
  return vysl;
}



int primprim(primka p1, primka p2, bod3d *prusecik)
{
  bod3d soucin,vektor1;
  float k,d,r,t;
  soucin=p1.u*p2.u;
  if((soucin.x < nepres) && (soucin.x > -nepres) &&
    (soucin.y < nepres) && (soucin.y > -nepres) &&
    (soucin.z < nepres) && (soucin.z > -nepres))
  {
    vektor1=p1.A-p2.A;
    soucin=p1.u*vektor1;
    if((soucin.x < nepres) && (soucin.x > -nepres) &&
      (soucin.y < nepres) && (soucin.y > -nepres) &&
      (soucin.z < nepres) && (soucin.z > -nepres))
    {

      return SHODNE;
    }
    else
      return ROVNOBEZNE;
  }
  else
  {

    d=p1.u.x*(-p2.u.y)+(p2.u.x*p1.u.y);
    if(d==0)
    {
      d=p1.u.x*(-p2.u.z)+(p2.u.x*p1.u.z);
      if(d==0)
      {
	d=p1.u.y*(-p2.u.z)+(p2.u.y*p1.u.z);
	if(d==0)
	{
	  return NECEKANA_CHYBA;
	}
	else
	{
	  t=((p1.A.y-p2.A.y)*(p2.u.z)+(p2.u.y)*(p2.A.z-p1.A.z))/d;
	  r=(p1.u.y*(p2.A.z-p1.A.z)-(p2.A.y-p1.A.y)*p1.u.z)/d;
	  if((p1.A.x-p2.A.x+p1.u.x*t-p2.u.x*r<nepres)&&
	     (p1.A.x-p2.A.x+p1.u.x*t-p2.u.x*r>-nepres))
	  {
	    prusecik->x=p1.A.x+p1.u.x*t;
	    prusecik->y=p1.A.y+p1.u.y*t;
	    prusecik->z=p1.A.z+p1.u.z*t;
	    return RUZNOBEZNE;
	  }
	  else
	    return MIMOBEZNE;
	}
      }
      else
      {
	t=((p1.A.x-p2.A.x)*(p2.u.z)+(p2.u.x)*(p2.A.z-p1.A.z))/d;
	r=(p1.u.x*(p2.A.z-p1.A.z)-(p2.A.x-p1.A.x)*p1.u.z)/d;
	if((p1.A.y-p2.A.y+p1.u.y*t-p2.u.y*r<nepres)&&
	  (p1.A.y-p2.A.y+p1.u.y*t-p2.u.y*r>-nepres))
	{
	  prusecik->x=p1.A.x+p1.u.x*t;
	  prusecik->y=p1.A.y+p1.u.y*t;
	  prusecik->z=p1.A.z+p1.u.z*t;
	  return RUZNOBEZNE;
	}
	else
	  return MIMOBEZNE;
      }
    }
    else
    {
      t=((p1.A.x-p2.A.x)*(p2.u.y) + (p2.u.x)*(p2.A.y-p1.A.y))/d;
      r=(p1.u.x*(p2.A.y-p1.A.y)-(p2.A.x-p1.A.x)*p1.u.y)/d;
      if((p1.A.z-p2.A.z+p1.u.z*t-p2.u.z*r<nepres)&&
	(p1.A.z-p2.A.z+p1.u.z*t-p2.u.z*r>-nepres))
      {
	prusecik->x=p1.A.x+p1.u.x*t;
	prusecik->y=p1.A.y+(p1.u.y*t);
	prusecik->z=p1.A.z+p1.u.z*t;
	return RUZNOBEZNE;
      }
      else
	return MIMOBEZNE;
    }
  }
}



int primrna(primka p,rovina r, bod3d *prusecik)
{
  float a,b,t;

  b=-((r.n.x*p.A.x)+(r.n.y*p.A.y)+(r.n.z*p.A.z)+r.d);
  a=r.n.x*p.u.x+r.n.y*p.u.y+r.n.z*p.u.z;
  if(a<nepres && a>-nepres)
    if(b<nepres && b>-nepres)
      return SHODNE;
    else return ROVNOBEZNE;
  else
  {
    t=b/a;
    (prusecik->x)=p.A.x+t*p.u.x;
    (prusecik->y)=p.A.y+t*p.u.y;
    (prusecik->z)=p.A.z+t*p.u.z;
    return RUZNOBEZNE;
  }
}




primka pricka(primka p1, primka p2)
{
  primka pric;
  rovina r;
  bod3d X;

  r.u=pric.u=p1.u*p2.u;
  r.v=p1.u;
  r.A=p1.A;
  r.n=r.u*r.v;
  r.d=-((r.n.x*r.A.x)+(r.n.y*r.A.y)+(r.n.z*r.A.z));
  primrna(p2,r,&pric.A);
  return pric;
}


float vzdal_bodu(bod3d A, bod3d B)
{
  return sqrt((A.x-B.x)*(A.x-B.x) + (A.y-B.y)*(A.y-B.y) + (A.z-B.z)*(A.z-B.z));
}


float norma(bod3d vektor)
{
  return sqrt(vektor.x*vektor.x+vektor.y*vektor.y+vektor.z*vektor.z);
}


bod3d urci_bod(bod3d X, bod3d vektor, float vzdalenost)
{
  bod3d vystup;
  bod3d v;
  float norma_v;

  norma_v=norma(vektor);
  v.x=(vektor.x/norma_v)*vzdalenost;
  v.y=(vektor.y/norma_v)*vzdalenost;
  v.z=(vektor.z/norma_v)*vzdalenost;
  return (X+v);
}


void najdi_krychle(primka p1, primka p2, krychle *krych1, krychle *krych2,
       krychle *krych3, krychle *krych4)
{
  primka pric;
  bod3d S,X;
  bod3d v1,v2,v3;
  float dpricky,strana,ca,cb,stranuhlop;

  pric=pricka(p1,p2);
  primprim(pric,p1,&S);
  primprim(pric,p2,&(krych1->K));
  dpricky=vzdal_bodu(S,krych1->K);
  strana=odm3pol*dpricky;
  ca=strana/odm3;
  stranuhlop=strana*sqrt(2)/2;
  cb=ca*2;

  krych2->K=krych1->K;
  krych1->L=urci_bod(S,p1.u,ca);
  krych1->R=urci_bod(S,-p1.u,cb);
  v1=krych1->L - krych1->K;
  v2=krych1->R - krych1->K;
  v3=v1*v2;
  krych1->Q=urci_bod(krych1->R,v1,strana);
  X=urci_bod(krych1->K,v2,stranuhlop);
  krych1->O=urci_bod(X,v3,stranuhlop);
  krych1->N=urci_bod(X,-v3,stranuhlop);
  krych1->P=urci_bod(krych1->O,v1,strana);
  krych1->M=urci_bod(krych1->N,v1,strana);

  krych2->L=urci_bod(S,-p1.u,ca);
  krych2->R=urci_bod(S,p1.u,cb);
  v1=krych2->L - krych2->K;
  v2=krych2->R - krych2->K;
  v3=v1*v2;
  krych2->Q=urci_bod(krych2->R,v1,strana);
  X=urci_bod(krych2->K,v2,stranuhlop);
  krych2->O=urci_bod(X,-v3,stranuhlop);
  krych2->N=urci_bod(X,v3,stranuhlop);
  krych2->P=urci_bod(krych2->O,v1,strana);
  krych2->M=urci_bod(krych2->N,v1,strana);

  krych3->K=krych4->K=S;
  S=krych1->K;
  krych3->L=urci_bod(S,p2.u,ca);
  krych3->R=urci_bod(S,-p2.u,cb);
  v1=krych3->L - krych3->K;
  v2=krych3->R - krych3->K;
  v3=v1*v2;
  krych3->Q=urci_bod(krych3->R,v1,strana);
  X=urci_bod(krych3->K,v2,stranuhlop);
  krych3->O=urci_bod(X,v3,stranuhlop);
  krych3->N=urci_bod(X,-v3,stranuhlop);
  krych3->P=urci_bod(krych3->O,v1,strana);
  krych3->M=urci_bod(krych3->N,v1,strana);

  krych4->L=urci_bod(S,-p2.u,ca);
  krych4->R=urci_bod(S,p2.u,cb);
  v1=krych4->L - krych4->K;
  v2=krych4->R - krych4->K;
  v3=v1*v2;
  krych4->Q=urci_bod(krych4->R,v1,strana);
  X=urci_bod(krych4->K,v2,stranuhlop);
  krych4->O=urci_bod(X,-v3,stranuhlop);
  krych4->N=urci_bod(X,v3,stranuhlop);
  krych4->P=urci_bod(krych4->O,v1,strana);
  krych4->M=urci_bod(krych4->N,v1,strana);

}

int zaokrouhli(float f)
{
  int i;
  float d;
  d=f-(float)((int)f);
  if(d>=0)
    i=(d>=0.5)?(int)(f+1):(int)f;
  else
    i=(-d>=0.5)?(int)(f-1):(int)f;
  return i;
}


krychle2d kosouhle_krychle(krychle vstup)
{
  krychle2d vystup;
  bod3d *uk=&vstup.K;
  bod2d *uk2=&vystup.K;

  for(int i=0;i<BODU_KRYCHLE;i++)
    *(uk2+i)=kosouhle(*(uk+i));
  return vystup;
}


int kontrola_primek(primka p1, primka p2, win *roz_ok)
{
  int i, max_y, TEXT_X,TEXT_Y, col;
  bod3d b;

  i=primprim(p1,p2,&b);
  if(i!=MIMOBEZNE)
  {
    switch (i) {
      case SHODNE:
	pis_chybu(roz_ok,SHODNE_PR);
	break;
      case ROVNOBEZNE:
	pis_chybu(roz_ok,ROVNOBEZNE_PR1,2);
	break;
      case RUZNOBEZNE:
	pis_chybu(roz_ok,RUZNOBEZNE_PR1,2);
	break;
    }
    setcolor(col);
  }
  return i;
}

